import { analyzeExport, convertToSpec } from '@knapsack/spec-utils';
import { analyzeComponent } from './utils/ng-component-analyzer.mjs';
import { importComponent } from './utils/importer.mjs';
function getComponentTypeInfo({ path, namedExport, resolveFrom, pkgPathAliases, }) {
    try {
        const { typeInfo } = analyzeExport({
            exportName: namedExport,
            fileName: path,
            pkgPathAliases,
            resolveFromDir: resolveFrom,
        });
        return typeInfo.type === 'class' &&
            typeInfo.prototype.typeInfo.type === 'object'
            ? typeInfo.prototype.typeInfo.properties
            : null;
    }
    catch (e) {
        // console.error(e);
        return null;
    }
}
export async function inferSpec({ path, namedExport, resolveFrom = process.cwd(), pkgPathAliases, }) {
    const typeInfo = getComponentTypeInfo({
        path,
        namedExport,
        resolveFrom,
        pkgPathAliases,
    });
    const component = await importComponent({
        path,
        alias: namedExport,
        resolveFrom,
        pkgPathAliases,
    });
    const { inputs, outputs, decoratorData, ngContentSelectors } = analyzeComponent(component);
    const props = {
        type: 'object',
        required: [],
        properties: {},
    };
    const slots = ngContentSelectors.reduce((acc, ngContentSelector) => {
        const slotName = ngContentSelector === '*' ? 'default' : ngContentSelector;
        acc[slotName] = {
            title: slotName,
        };
        return acc;
    }, {});
    const extraDocs = [];
    const { selector, standalone: isStandalone } = decoratorData;
    extraDocs.push({
        id: 'overview',
        title: 'Overview',
        description: '',
        tableRows: [
            {
                Title: 'Standalone Component',
                Description: isStandalone ? 'Yes' : 'No',
            },
            {
                Title: 'Selector',
                // Preserving the "`" for markdown formatting of code
                Description: `\`<${selector}>\``,
            },
            {
                Title: 'Package',
                Description: `\`${path}\``,
            },
            {
                Title: 'Named Export',
                Description: `\`${namedExport}\``,
            },
        ],
    });
    const propsTableRows = [];
    inputs.forEach((input) => {
        var _a;
        const { propName, templateName, required } = input;
        const propInfo = typeInfo === null || typeInfo === void 0 ? void 0 : typeInfo[propName];
        if (propInfo) {
            const { prop } = convertToSpec({
                typeInfo: propInfo.typeInfo,
                trustRequiredProps: false,
            });
            if (prop) {
                props.properties[propName] = prop;
                if ((_a = propInfo.jsDoc) === null || _a === void 0 ? void 0 : _a.description) {
                    props.properties[propName].description = propInfo.jsDoc.description;
                }
                if (required) {
                    props.required.push(propName);
                }
            }
            const { typeInfo: { tsRawType }, } = propInfo;
            propsTableRows.push({
                'Input Name': `\`${propName}\``,
                'Template Name': `\`${templateName}\``,
                Type: `\`${tsRawType}\``,
            });
        }
    });
    if (propsTableRows.length) {
        extraDocs.push({
            id: 'inputs',
            title: 'Inputs',
            description: '',
            tableRows: propsTableRows,
        });
    }
    if (outputs.length) {
        extraDocs.push({
            id: 'outputs',
            title: 'Outputs',
            description: '',
            tableRows: outputs.map(({ propName, templateName }) => ({
                'Output Name': `\`${propName}\``,
                'Template Name': `\`${templateName}\``,
            })),
        });
    }
    if (Object.keys(slots).length) {
        extraDocs.push({
            id: 'slots',
            title: 'Slots',
            description: '',
            tableRows: Object.entries(slots).map(([slotName, slot]) => ({
                'Slot Name': `\`${slotName}\``,
                'Slot Title': slot.title,
            })),
        });
    }
    return {
        ksTemplateSpec: {
            isInferred: true,
            props,
            slots,
            extraDocs,
            hidePropsTable: true,
            hideSlotsTable: true,
        },
        inputs,
        outputs,
    };
}
//# sourceMappingURL=infer-spec.mjs.map