import '@angular/compiler';
import { isObject } from '@knapsack/utils';
import { analyzeComponent, } from './ng-component-analyzer.mjs';
const buildTemplate = ({ innerTemplate, inputs, outputs, selector, }) => {
    // https://www.w3.org/TR/2011/WD-html-markup-20110113/syntax.html#syntax-elements
    const voidElements = [
        'area',
        'base',
        'br',
        'col',
        'command',
        'embed',
        'hr',
        'img',
        'input',
        'keygen',
        'link',
        'meta',
        'param',
        'source',
        'track',
        'wbr',
    ];
    const firstSelector = selector.split(',')[0];
    const templateReplacers = [
        [/(^.*?)(?=[,])/, '$1'],
        [/(^\..+)/, 'div$1'],
        [/(^\[.+?])/, 'div$1'],
        [/([\w[\]]+)(\s*,[\w\s-[\],]+)+/, `$1`],
        [/#([\w-]+)/, ` id="$1"`],
        [/((\.[\w-]+)+)/, (_, c) => ` class="${c.split `.`.join ` `.trim()}"`],
        [/(\[.+?])/g, (_, a) => ` ${a.slice(1, -1)}`],
        [
            /([\S]+)(.*)/,
            (template, elementSelector) => {
                return voidElements.some((element) => elementSelector === element)
                    ? template.replace(/([\S]+)(.*)/, `<$1$2${inputs} ${outputs} />`)
                    : template.replace(/([\S]+)(.*)/, `<$1$2${inputs} ${outputs}>\n${innerTemplate}\n</$1>`);
            },
        ],
    ];
    return templateReplacers.reduce((prevSelector, [searchValue, replacer]) => prevSelector.replace(searchValue, replacer), firstSelector);
};
/**
 * Check if the name matches the criteria for a valid identifier.
 * A valid identifier can only contain letters, digits, underscores, or dollar signs.
 * It cannot start with a digit.
 */
const isValidIdentifier = (name) => /^[a-zA-Z_$][a-zA-Z0-9_$]*$/.test(name);
/**
 * Returns the property name, if it can be accessed with dot notation. If not,
 * it returns `this['propertyName']`.
 */
export const formatPropInTemplate = (propertyName) => isValidIdentifier(propertyName) ? propertyName : `this['${propertyName}']`;
const separateInputsOutputsAttributes = (ngComponentInputsOutputs, props = {}) => {
    const inputs = ngComponentInputsOutputs.inputs
        .filter((i) => i.templateName in props)
        .map((i) => i.templateName);
    const outputs = ngComponentInputsOutputs.outputs
        .filter((o) => o.templateName in props)
        .map((o) => o.templateName);
    return {
        inputs,
        outputs,
        otherProps: Object.keys(props).filter((k) => ![...inputs, ...outputs].includes(k)),
    };
};
function stringifyObject(obj) {
    return JSON.stringify(obj).replace(/"/g, "'");
}
/**
 * Converts a component into a template with inputs/outputs present in initial props
 * @param component
 * @param initialProps
 * @param innerTemplate
 */
export const computesTemplateFromComponent = async ({ component, initialProps, innerTemplate = '', }) => {
    const { inputs, outputs, decoratorData: ngComponentMetadata, } = analyzeComponent(component);
    if (!(ngComponentMetadata === null || ngComponentMetadata === void 0 ? void 0 : ngComponentMetadata.selector)) {
        // @todo: remove this when we have a better way to handle this
        // Allow to add renderer component when NgComponent selector is undefined
        return `<ng-container *ngComponentOutlet="storyComponent"></ng-container>`;
    }
    const { inputs: initialInputs, outputs: initialOutputs } = separateInputsOutputsAttributes({ inputs, outputs }, initialProps);
    function shouldValueBeEvaluated({ value }) {
        if (typeof value === 'string')
            return false;
        return true;
    }
    const templateInputs = initialInputs.length > 0
        ? ` ${initialInputs
            .map((i) => {
            const value = initialProps === null || initialProps === void 0 ? void 0 : initialProps[i];
            if (shouldValueBeEvaluated({ value })) {
                if (Array.isArray(value)) {
                    return `[${i}]="${stringifyObject(value)}"`;
                }
                if (isObject(value)) {
                    return `[${i}]="${stringifyObject(value)}"`;
                }
                return `[${i}]="${value}"`;
            }
            return `${i}=${JSON.stringify(value)}`;
        })
            .join(' ')}`
        : '';
    const templateOutputs = outputs
        // calls `this.logIt()` in wrapper component
        .map((i) => `(${i.propName})="logIt('${i.propName}')"`)
        .join(' ');
    return buildTemplate({
        selector: ngComponentMetadata.selector,
        innerTemplate,
        inputs: templateInputs,
        outputs: templateOutputs,
    });
};
/**
 * Stringify an object with a placeholder in the circular references.
 */
function stringifyCircular(obj) {
    const seen = new Set();
    return JSON.stringify(obj, (key, value) => {
        if (typeof value === 'object' && value !== null) {
            if (seen.has(value)) {
                return '[Circular]';
            }
            seen.add(value);
        }
        return value;
    });
}
const createAngularInputProperty = ({ propertyName, value, }) => {
    let templateValue;
    if (typeof value === 'string') {
        templateValue = `'${value}'`;
    }
    else if (isObject(value)) {
        templateValue = stringifyCircular(value)
            .replace(/'/g, '\u2019')
            .replace(/\\"/g, '\u201D')
            .replace(/"([^-"]+)":/g, '$1: ')
            .replace(/"/g, "'")
            .replace(/\u2019/g, "\\'")
            .replace(/\u201D/g, "\\'")
            .split(',')
            .join(', ');
    }
    else {
        // @ts-expect-error will end up invoking it's `toString` method if it has it
        templateValue = value;
    }
    return `[${propertyName}]="${templateValue}"`;
};
/**
 * Converts a component into a template with inputs/outputs present in initial props
 * @param component
 * @param initialProps
 * @param innerTemplate
 */
export const computesTemplateSourceFromComponent = ({ component, initialProps, }) => {
    const { inputs, outputs, decoratorData: ngComponentMetadata, } = analyzeComponent(component);
    if (!ngComponentMetadata) {
        return null;
    }
    if (!ngComponentMetadata.selector) {
        // Allow to add renderer component when NgComponent selector is undefined
        return `<ng-container *ngComponentOutlet="${component.name}"></ng-container>`;
    }
    const { inputs: initialInputs, outputs: initialOutputs } = separateInputsOutputsAttributes({ inputs, outputs }, initialProps);
    const templateInputs = initialInputs.length > 0
        ? ` ${initialInputs
            .map((propertyName) => createAngularInputProperty({
            propertyName,
            value: initialProps === null || initialProps === void 0 ? void 0 : initialProps[propertyName],
            // argType: argTypes?.[propertyName],
        }))
            .join(' ')}`
        : '';
    const templateOutputs = initialOutputs.length > 0
        ? ` ${initialOutputs
            .map((i) => `(${i})="${formatPropInTemplate(i)}($event)"`)
            .join(' ')}`
        : '';
    return buildTemplate({
        selector: ngComponentMetadata.selector,
        innerTemplate: '',
        inputs: templateInputs,
        outputs: templateOutputs,
    });
};
//# sourceMappingURL=computes-template-from-component.mjs.map